local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
if not RW then return end
local L = LibStub("AceLocale-3.0"):GetLocale("RW-2.0")

local Party
local math_modf = math.modf

RW.defaults = {
	profile = {
		Font = "accid",
		FontSize = 16,
		FontOutline = "OUTLINE",
		FontColor = {0.47, 0.75, 1, 1},
		FontAlign = "LEFT",
		BGTexture = "Solid",
		BGColor = {0, 0, 0, 0.5},
		Border = "IshBorder",
		BorderColor = {0.15, 0.24, 0.31, 1},
		BorderSize = 12,
		BarTexture = "HalD",
		BarColor = {0.22, 0.34, 0.46, 1},
		Insets = 1,
		BorderMargin = 5
	}
}

function RW:InTable(t, entry)
	for i, v in pairs(t) do
		if v == entry then return true end
	end
end

function RW:InTable2(t, key, entry)
	for i, v in pairs(t) do
		if v[key] == entry then return true end
	end
end

function RW:GetTableData(t, key, entry)
	for i, v in pairs(t) do
		if v[key] == entry then return v end
	end
end

function RW:GetIndexForvalue(t, value)
	for i, v in pairs(t) do
		if v == value then return i end
	end
end

function RW:NrElements(t)
	local count = 0
	for i, v in pairs(t) do
		count = count + 1
	end
	return count
end

function RW:TRemove(t, entry)
	for i, v in pairs(t) do
		if v == entry then 
			tremove(t, i)
		end
	end
end

function RW:CopyTable(t, d)
	if (d  or 0) > 5 then return nil end
	local new = {}
	for i, v in pairs(t) do
		if type(v) == "table" then
			new[i] = self:CopyTable(v, (d or 0) + 1)
		else
			new[i] = v
		end
	end
	return new
end

function RW:TimeToStr(time)
	if type(time) ~= "number" then return time end
	time = math.floor(time)
	local t
	if time < 60 then
		return L.COMBATTIME_SEC:format(time)
	elseif time % 60 == 0 then
		return L.COMBATTIME_MIN:format(time/60)
	else
		return L.COMBATTIME:format(time/60, time % 60)
	end
end

function RW:TimeToStringShort(t)
	if type(t) ~= "number" then return t end
	if t <= 10 then
		return ("%.1fs"):format(t)
	elseif t <= 60 then
		return ("%ds"):format(t)
	else
		return ("%0.2d:%0.2d"):format(t/60, math.fmod(t, 60))
	end
end

function RW:RGBConvert(r, g, b, a)
	return r/255, g/255, b/255, (a and a/255 or 1)
end

function RW:Lerp(s, e, a)
	if not s or not e or not a then return 0 end
	return s + ((e - s) * a)
end

function RW:AddBit(t, b)
	if bit.band(t or 0, b or 0) ~= b then t = (t or 0) + (b or 0) end
	return (t or 0)
end

function RW:RemoveBit(t, b)
	if bit.band(t or 0, b or 0) == b then t = (t or 0) - (b or 0) end
	return (t or 0)
end

function RW:ColorNames(str)
	if not Party then Party = RW:GetPlugin("Party") end
	str = str:gsub(">([%S\s-]+)<", function(name)
		if Party.group[name] then
			local color = RAID_CLASS_COLORS[Party.group[name].class]
			if color then
				name = ("|cff%.2x%.2x%.2x%s|r"):format(color.r * 255, color.g * 255, color.b * 255, name)
			end
		end
		return name
	end)
	return str
end

-- A bit of hack to solve if the first argument is correct but the rest is not, 
-- only for timer and message formating realy.
function RW:Format(text, ...)
	local ok, str = pcall(format, text, ...)
	if not ok then
		local first = select(1, ...)
		ok, str = pcall(format, text, first, L.UNKNOWN, 0)
	end
	if not ok then
		local first = select(1, ...)
		ok, str = pcall(format, text, first, 0)
	end
	return ok and str or text:gsub("(%%.)", L.UNKNOWN)
end

do
	local playerSpells
	function RW:GetRandomSpell()
		if not playerSpells then
			playerSpells = {}
			local i = 1
			while true do
				local spellName = GetSpellName(i, BOOKTYPE_SPELL)
				if not spellName then break end
				playerSpells[#playerSpells + 1] = spellName
				i = i + 1
			end
		end
		local name, rank, icon = GetSpellInfo(playerSpells[math.random(1, #playerSpells)])
		return name, rank, icon
	end
end

-- Tekkub's ColorGradient() http://www.wowwiki.com/ColorGradient
function RW:ColorGradient(perc, r1, g1, b1, r2, g2, b2, r3, g3, b3)
	if perc >= 1 then return r3, g3, b3 elseif perc <= 0 then return r1, g1, b1 end
	local segment, relperc = math_modf(perc*2)
	if segment == 1 then r1, g1, b1, r2, g2, b2 = r2, g2, b2, r3, g3, b3 end
	return r1 + (r2-r1)*relperc, g1 + (g2-g1)*relperc, b1 + (b2-b1)*relperc
end

-----------------------------
-- Party functions
function RW:IsHeroic()
	local name, instanceType, difficultyIndex, difficultyName, maxPlayers, playerDifficulty, isDynamic = GetInstanceInfo()
	if (instanceType == "party" and difficultyIndex == 2) or difficultyIndex > 2 then return true end
	if instanceType == "none" then return false end
end

function RW:InstanceSize()
	local _, instanceType, difficulty, _, maxPlayers = GetInstanceInfo()
	if instanceType == "none" then return 10 end
	return maxPlayers
end

function RW:CreateAnchor(name, parent, text)
	local frame = CreateFrame("Frame", name or nil, parent or nil)
	frame:SetPoint("CENTER")
	frame:SetWidth(200)
	frame:SetHeight(25)
	frame:EnableMouse(true)
	frame:RegisterForDrag("LeftButton")
	frame:SetMovable(true)

	local bg = frame:CreateTexture(nil, "PARENT")
	bg:SetAllPoints(frame)
	bg:SetTexture(0.39, 0.55, 0.87, 0.7)
	frame.background = bg
	
	local str = frame:CreateFontString(name and name.."Text", "ARTWORK")
	str:SetFont(LibStub("LibSharedMedia-3.0"):Fetch("font", "accid"), 16, "OUTLINE")
	str:SetAllPoints()
	str:SetText(text)
	
	frame.str = str
	frame:Hide()
	return frame
end

function RW:CreateBar(rightClickFunc)
	local db = self.defaults.profile
	local frame = CreateFrame("Frame", "RW_Bar", UIParent)
	local bar = CreateFrame("StatusBar", frame:GetName().."Bar", frame)
	local timeText = bar:CreateFontString(frame:GetName().."TimeText", "ARTWORK")
	local text = bar:CreateFontString(frame:GetName().."Text", "ARTOWRK")
	local icon = frame:CreateTexture(frame:GetName().."Icon", "ARTWORK")
	
	frame:SetPoint("CENTER")
	frame:SetWidth(255)
	bar:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -db.BorderMargin, db.BorderMargin)
	bar:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -db.BorderMargin, -db.BorderMargin)
	icon:SetPoint("TOPLEFT", frame, "TOPLEFT", db.BorderMargin, -db.BorderMargin)
	icon:SetPoint("BOTTOMRIGHT", bar, "BOTTOMLEFT", -2, 0)
	timeText:SetPoint("RIGHT", bar, "RIGHT", -4, 0)
	text:SetPoint("LEFT", bar, "LEFT", 2, 0)
	text:SetPoint("RIGHT", timeText, "LEFT")
	
	bar:SetMinMaxValues(0,1)
	
	timeText:SetJustifyH("LEFT")
	timeText:SetWidth(45)
	
	frame.timeText = timeText
	frame.text = text
	frame.bar = bar
	frame.icon = icon
	
	frame.rightClickfuc = rightClickFunc
	
	frame:SetScript("OnEvent", function(self, event, arg1, arg2)
		if arg1 == "LSHIFT" or arg1 == "RSHIFT" then
			if arg2 == 1 then
				frame:EnableMouse(true)
				self:SetScript("OnMouseDown", self.rightClickfuc)
			else
				frame:EnableMouse(false)
				self:SetScript("OnMouseDown", nil)
			end
		end
	end)
	frame:Hide()
	return frame
end

---------------------
-- AceGUI Helpers
function RW.SetGUIOption(widget, event, ...)
	local db = widget:GetUserData("db")
	local key = widget:GetUserData("key")
	
	if select("#", ...) > 1 then
		for i = 1, select("#", ...) do
			db[key][i] = select(i, ...)
		end
	else
		db[key] = ...
	end
end

function RW:PrepareWidget(widget, db, key)
	widget:SetUserData("db", db)
	widget:SetUserData("key", key)
	widget:SetCallback("OnValueChanged", RW.SetGUIOption)
end

------------------------
-- Map distance stuff

-- Credits to Kollektiv and DXE letting me use the map dimensions from DXE
-- http://www.wowace.com/addons/deus-vox-encounters/
-------------------------
-- MAP DIMENSIONS
-------------------------

local MapDims= {
	Ulduar = {
		[1] = {w = 3064.9614761023, h = 2039.5413309668}, 	-- Expedition Base Camp
		[2] = {w = 624.19069622949, h = 415.89374357805}, 	-- Antechamber of Ulduar
		[3] = {w = 1238.37427179,   h = 823.90183235628}, 	-- Conservatory of Life
		[4] = {w = 848.38069183829, h = 564.6688835337}, 	-- Prison of Yogg-Saron
		[5] = {w = 1460.4694647684, h = 974.65312886234},  -- Spark of Imagination
		[6] = {w = 576.71549337896, h = 384.46653291368},  -- The Mind's Eye (Under Yogg)
	},
	Naxxramas = {
		[1] = {w = 1018.3655494957, h = 679.40523953718}, -- Construct
		[2] = {w = 1019.1310739251, h = 679.18864376555}, -- Arachnid
		[3] = {w = 1118.1083638787, h = 744.57895516418}, -- Military
		[4] = {w = 1117.0809918236, h = 745.97398439776}, -- Plague
		[5] = {w = 1927.3190541014, h = 1284.6530841959}, -- Entrance
		[6] = {w = 610.62737087301, h = 407.3875157986},  -- KT/Sapphiron
	},
	TheObsidianSanctum = {
		[0] = {w = 1081.6334214432, h = 721.79860069158},
	},
	TheEyeofEternity = {
		[1] = {w = 400.728405332355, h = 267.09113174487},
	},
	TheArgentColiseum = {
		[1] = {w = 344.20785972537, h = 229.57961178118},
		[2] = {w = 688.60679691348, h = 458.95801567569},
	},
	VaultofArchavon = {
		[1] = {w = 842.2254908359, h = 561.59878021123},
	},
	IcecrownCitadel = {
		[1] = {w = 1262.8025621533, h = 841.91669450207}, -- The Lower Citadel
		[2] = {w = 993.25701607873, h = 662.58829476644}, -- The Rampart of Skulls
		[3] = {w = 181.83564716405, h = 121.29684810833}, -- Deathbringer's Rise
		[4] = {w = 720.60965618252, h = 481.1621506613}, -- The Frost Queen's Lair
		[5] = {w = 1069.6156745738, h = 713.83371679543}, -- The Upper Reaches
		[6] = {w = 348.05218433541, h = 232.05964286208}, -- Royal Quarters
		-- [7] = {w = 0, h = 0},
	},
	Dalaran = {
		[1] = {w = 829.97011173593, h = 553.36899735997}
	}
}

function RW:GetDistanceToUnit(uID)
	if WorldMapFrame:IsShown() then
		return
	end
	
	local px, py = GetPlayerMapPosition("player")
	if px == 0 and py == 0 then 
		SetMapToCurrentZone()
		px, py = GetPlayerMapPosition("player")
		if px == 0 and py == 0 then return end
	end
	
	if not UnitIsConnected(uID) then return end

	local dimList = MapDims[GetMapInfo()]
	
	if not dimList then return end
	local dim = dimList[GetCurrentMapDungeonLevel()]
	if not dim then
		SetMapToCurrentZone()
		dim = dimList[GetCurrentMapDungeonLevel()]
		if not dim then
			dim = dimList[GetCurrentMapDungeonLevel()]
			if not dim then return end
		end
	end

	local tx, ty = GetPlayerMapPosition(uID)
	if px == 0 and py == 0 then return end
	if tx == 0 and ty == 0 then return end
	
	local dx = (px - tx) * dim.w
	local dy = (py - ty) * dim.h
	
	local range = math.sqrt(dx * dx + dy * dy)
	
	return range, dx, dy
end

function RW:GetDistanceToPosition(x, y)	
	if WorldMapFrame:IsShown() then
		return
	end
	
	if px == 0 and py == 0 then 
		SetMapToCurrentZone()
		px, py = GetPlayerMapPosition("player")
		if px == 0 and py == 0 then return end
	end
	
	local dimList = MapDims[GetMapInfo()]
	
	if not dimList then return end
	local dim = dimList[GetCurrentMapDungeonLevel()]
	if not dim then
		SetMapToCurrentZone()
		dim = dimList[GetCurrentMapDungeonLevel()]
		if not dim then 
			dim = dimList[GetCurrentMapDungeonLevel()]
			if not dim then return end
		end
	end
	
	local tx, ty = x, y
	if px == 0 and py == 0 then return end
	if tx == 0 and ty == 0 then return end
	
	local dx = (px - tx) * dim.w
	local dy = (py - ty) * dim.h
	
	local range = math.sqrt(dx * dx + dy * dy)
	
	return range, dx, dy
end

function RW:GetPlayerPosition()
	local px, py = GetPlayerMapPosition("player")
	
	if px == 0 and py == 0 then 
		SetMapToCurrentZone()
		px, py = GetPlayerMapPosition("player")
		if px == 0 and py == 0 then return end
	end

	local dimList = MapDims[GetMapInfo()]
	
	if not dimList then return end
	local dim = dimList[GetCurrentMapDungeonLevel()]
	if not dim then
		SetMapToCurrentZone()
		dim = dimList[GetCurrentMapDungeonLevel()]
		if not dim then
			dim = dimList[GetCurrentMapDungeonLevel()]
			if not dim then return end
		end
	end
	
	return px * dim.w, py * dim.h
end


function RW:GetMapDims(zone)
	zone = zone or GetMapInfo()
	
	local dimList = MapDims[zone]
	
	if not dimList then return end
	local dim = dimList[GetCurrentMapDungeonLevel()]
	if not dim then
		SetMapToCurrentZone()
		dim = dimList[GetCurrentMapDungeonLevel()]
		if not dim then
			dim = dimList[GetCurrentMapDungeonLevel()]
			if not dim then return end
		end
	end
	
	return dim
end
